function [CRLB,mdat] = CRLB_NQ_with_pileup(spectran,eta, varargin)
% function [CRLB,mdat] = CRLB_NQA_with_pileup(spectran,Tintegrate,deadtime, varargin)
%  Computes Cramer-Rao lower bound (CRLB) for log NQA data with pileup 
%  directly from general formula for Fisher Info matrix
%  for general Gaussian case (Eq. 3.31 of Kay). See Onenote with keyword CRLB for formula
% Also assumes counting detectors with pileup using zero pulse width approximation
%    note that for non-zero eta stats are not Poisson
%  inputs: 
%    spectran:  struct of spectrum transmitted through object. Required fields
%        specnum: the photon number spectrum. Integral is total counts during measurement period
%        egys: (keV) the x-ray energies for each entry of specnum
%        mus: matrix of samples of the basis set functions at egys. Each function is separate column%
%     eta: counts per dead time
%     dA_offset: (optional = 0.001*ones(1,nbasis))  the A-vector step offsets for derivative calculation
%  outputs:
%    CRLB: the CRLB with pileup
%    mdat: debugging info struct. Has workspace at end of executing the function
%  REA 10/1/10 - 4/Feb/2015 10:32
% COPYRIGHT:   Aprend Technology, 2011-2015. All rights reserved.   

   % get nbasis parameter for use by local funcs
[spectran.negys,spectran.nbasis] = size(spectran.mus);
dA_offset = 0.001*ones(1,spectran.nbasis);  % the step offsets for derivative calculation

nreqargs = 2;
if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'da_offset';         dA_offset=varargin{i+1};   assert(numel(dA_offset)==spectran.nbasis);  i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end

dAs = diag(dA_offset);

F2 = zeros(spectran.nbasis); % 2nd term of Fisher information matrix
   % get derivative of covariance and log NQ covariance
[dRdA, RL] = dRdALOC(spectran, eta,dAs);
RLinv = inv(RL);
   % first accumulate the derivative of covariance terms
for i = 1:2
   for j= 1:2
     F2(i,j) = 0.5*trace(RLinv*dRdA(:,:,i)*RLinv*dRdA(:,:,j));
   end
end
M = M_NQ_with_pileup(spectran,eta,dAs);
F1 = M'*RLinv*M;
F = F1 + F2;
CRLB = inv(F);
if nargout >1 % save workspace
    CRLB_const_cov = inv(F1);
    mdat = GetWorkspace;
end

%------------------------------ LOCAL FUNCTIONS ----------------------------------
function [dRdA, cv0] = dRdALOC(specdat, eta ,dAs)
nbins = 2;
dRdA = zeros(nbins,nbins,2); % 2 planes, one for each A-vector component
cv0 = covLOC(specdat, eta);
for kdim = 1:2 
    sp2 = specdat;
    sp2.specnum = exp(-specdat.mus*dAs(kdim,:)').*specdat.specnum; % note transpose
    cv = covLOC(sp2, eta);
    dRdA(:,:,kdim) = (cv - cv0)/dAs(kdim,kdim);
end

     %------------
function RL = covLOC(specdat, eta)
    % extract rate from integral of spectrum and integrate time
N = trapz(specdat.egys,specdat.specnum);
    % normalize the input spectrum
specdat.specnum_norm = specdat.specnum/N;
Ebar = trapz(specdat.egys,specdat.egys(:).*specdat.specnum_norm(:));
E2bar = trapz(specdat.egys,specdat.egys(:).^2.*specdat.specnum_norm(:));
F = E2bar/(Ebar)^2;
eta1 = 1 + eta;
RL = (1/N)*[1/eta1,1/eta1;1/eta1, F] ; % covariance of log with deadtime
   % see Table I of SNR with pileup paper

     %------------
function M = M_NQ_with_pileup(specdat,eta,dAs)
M = zeros(2); % will hold results
N = trapz(specdat.egys,specdat.specnum); % number of photons
specdat.specnum_norm = specdat.specnum/N;
eta1 = 1 + eta;
N0rec = N/eta1;
Q0 = trapz(specdat.egys,specdat.egys(:).*specdat.specnum(:));  
for kdim = 1:2 
    spec2 = exp(-specdat.mus*dAs(kdim,:)').*specdat.specnum_norm; % note transpose
    frac = trapz(specdat.egys,spec2); % fraction of incident spectrum after dA step
    eta2 = frac*eta; % count rate and therefore eta decreases because of added attenuation
    N2rec = (frac*N)/(1+eta2);
    M(1,kdim) = -(log(N2rec) - log(N0rec))/dAs(kdim,kdim);
         % Q
    Q2 = N*trapz(specdat.egys,specdat.egys(:).*spec2(:)); % multiply by N to un-normalize spec2
    M(2,kdim) = -(log(Q2) - log(Q0))/dAs(kdim,kdim);
end