% P54SNRpileup5_1.m Monte Carlo Validation
%
% COPYRIGHT:  Aprend Technology and Robert E. ALvarez, 2015. All rights reserved.
 
%% init vars
post_title = 'P54SNRpileup5_1';
% modify this for your computer
dirname = ['L:\Projects\Blog\Posts\' post_title '\'];
addpath([dirname 'code']);
% MakeBlogPostFileLists(post_title);

%% Create the input spectrum and initialize parameters
tube_voltage = 120; % keV  
[~,~,specdat] = XrayTubeSpectrumTasmip(tube_voltage,'number_spectrum','clipzeros');
specdat.mus = MusFromMaterials({'soft_tissue','bone_cortical'},specdat.egys);
   % filter spectrum through object
thickness_object = 20;
a_object = [1; 0]; % 'soft_tissue','bone_cortical' for object--note soft tissue only
specdat.specnum = specdat.specnum.*exp(-specdat.mus*(thickness_object*a_object));
specdat.specnum_norm = specdat.specnum/trapz(specdat.egys,specdat.specnum);
spinv =  SpectrumCumulativeInverse(specdat);
nbins = 4;
[specdat.idx_threshold,specdat.Ethresholds] = OptimNbinsThresholds(specdat,nbins);
egy_bin_edges =  [0 specdat.Ethresholds' inf];
    % set up for log of no-noise NQ and PHA data for no object (air)
    % for PHA fractions per bin, 
    %  use that OptimNbinsThresholds sets bin counts equal
fracs4PHA = ones(1,nbins)/nbins; 
   % for NQ use <Q> = <nphotons>*Ebar
Ebar = trapz(specdat.egys,specdat.specnum_norm.*specdat.egys);   
    % init etas
etas = [0 .25 0.5]; 
nphotons2do = geospace(500,2500,5);
mean_cps = 1e6; % 1/seconds

%% generate MC data and theoretical values from formulas
n_nphotons2do = numel(nphotons2do);
n_etas = numel(etas);
RA_NQ_all = zeros(2,2,n_etas,n_nphotons2do); % 2 = number basis funcs
RA_NQ_all_se = zeros(2,2,n_etas,n_nphotons2do);
RA_PHA_all = zeros(2,2,n_etas,n_nphotons2do);
RA_PHA_all_se = zeros(2,2,n_etas,n_nphotons2do); % standard deviation of mean
CRLB_PHA_all = zeros(2,2,n_etas,n_nphotons2do);
CRLB_NQ_all = zeros(2,2,n_etas,n_nphotons2do);
nMCtrials = 5000; %number of trials to use for getting A vector variances
nbootstrap = 100; % number of boot strap trials for standard error calc
ToRow = @(x) x(:)'; % function to use for bootstrap calcs
fprintf('starting compute MC data ...\n');
for keta = 1:n_etas
    fprintf('keta %d\n', keta)
    eta = etas(keta);
    deadtime = eta/mean_cps;    
    for kt = 1: n_nphotons2do
        nphotons = nphotons2do(kt);
        t_integrate = nphotons/mean_cps;
        specdat.specnum = nphotons*specdat.specnum_norm;
             % generate random NQ and PHA data
             % first, calc number of pulses to generate
             % to make sure do not "run off end"
        npulses2gen = ceil( nphotons + 10*sqrt(nphotons) );
           % preallocate matrices for results
        nqdat = zeros(nMCtrials,2);
        phadat = zeros(nMCtrials,nbins);
        for ktrial = 1:nMCtrials
            dt_pulse = -(1/mean_cps)*log(rand(npulses2gen,1)); 
                % exponentially distributed interpulse times
            [npulses_record, egys_rec] = ...
                PileupPulses(dt_pulse,deadtime, t_integrate,'specinv',spinv);
            nqdat(ktrial, :) = [npulses_record(1), sum(egys_rec)]; 
                % PHA data
            counts = histc(egys_rec,egy_bin_edges);
            phadat(ktrial,:) = counts(1:(end-1));
        end
        assert(all(nqdat(:,1)>0) && all(phadat(:)>0));
             % compute A vectors from NQ random data using linear MLE
           % compute linear MLE for NQ and PHA
        [~, nqstats] = CRLB_NQ_with_pileup(specdat,eta);
        MLE_NQ = nqstats.CRLB_const_cov*nqstats.M'*nqstats.RLinv;
        [~, phastats] = CRLB_PHA_with_pileup(specdat,eta);
        MLE_PHA = phastats.CRLB_const_cov*phastats.M'*phastats.RLinv;

        LsNQ = -bsxfun(@minus,log(nqdat),log([nphotons, nphotons*Ebar]));
        As_NQ_calc = (MLE_NQ*LsNQ')'; % use transposes since MLE expects column vector of data
        bootstatNQ = bootstrp(nbootstrap,@(x) ToRow(cov(x)),As_NQ_calc);
        RA_NQ_mc = reshape(mean(bootstatNQ),2,2);
        RA_NQ_mc_se = reshape(std(bootstatNQ),2,2);
        RA_NQ_all(:,:,keta,kt) = RA_NQ_mc;
        RA_NQ_all_se(:,:,keta,kt) = RA_NQ_mc_se;
        CRLB_NQ_all(:,:,keta,kt) = nqstats.CRLB;

        LsPHA = -bsxfun(@minus,log(phadat),log(nphotons*fracs4PHA));
        As_PHA_calc = (MLE_PHA*LsPHA')'; % use transposes since MLE expects column vector of data
        bootstatPHA = bootstrp(nbootstrap,@(x) ToRow(cov(x)),As_PHA_calc);
        RA_PHA_mc = reshape(mean(bootstatPHA),2,2);
        RA_PHA_mc_se = reshape(std(bootstatPHA),2,2);
        RA_PHA_all(:,:,keta,kt) = RA_PHA_mc;
        RA_PHA_all_se(:,:,keta,kt) = RA_PHA_mc_se;
        CRLB_PHA_all(:,:,keta,kt) = phastats.CRLB;
        fprintf('.');
    end
    fprintf('%d of %d\n',keta,n_etas)
end 

%% plot MC variance and CRLB vs number of photons and eta--with pileup
fh = figure;
linewid = 1.5;
colors = 'krbgm';
plot_symbols = {'o','x','*','d','+'};
plot_order = n_etas:-1:1; % use this so legends in same order as plots
eta_strings = strtrim(cellstr(num2str(etas(plot_order(:))','%4.2f')));
hax = [];
min_photons = min(nphotons2do) ;
max_photons = max(nphotons2do);
    % NQ variance
max_y = 10;
xticks = [min_photons 1000 max_photons]; 
yticks = [.1 1 10];   
for kplot = 1:2
    subplot(2,2,kplot)
    hold on
    for keta = plot_order
        plot(nphotons2do,squeeze(RA_NQ_all(kplot,kplot,keta,:)),[plot_symbols{keta} colors(keta)], ...
            'linewidth',1.5);
    end
    axis([min_photons max_photons 0 max_y])
    set(gca,'xscale','log','yscale','log')
    if kplot==1
        hy = ylabel('NQ-variance','fontsize',13);
        ht1 = text(mean(xticks),0.5*yticks(end),sprintf('NQ\nA_{%d}',kplot),'fontsize',14);
        set(gca,'ytick',yticks)
    else
        ht1 = text(mean(xticks),0.02*yticks(end),sprintf('NQ\nA_{%d}',kplot),'fontsize',14);
        hl = legend(eta_strings,'location','northeast');
    end
    for keta = plot_order
        plot(nphotons2do,squeeze(CRLB_NQ_all(kplot,kplot,keta,:)),['-' colors(keta)],'linewidth',1.5);
    end
    hold off
    set(gca,'fontsize',14,'linewidth',1.5,'xtick',xticks)
    hax(end+1) = gca;
end
%    PHA variance 
for kplot = 1:2
    subplot(2,2,kplot+2)
    hold on
    for keta = plot_order
        plot(nphotons2do,squeeze(RA_PHA_all(kplot,kplot,keta,:)),[plot_symbols{keta} colors(keta)],'linewidth',1.5);
        plot(nphotons2do,squeeze(CRLB_PHA_all(kplot,kplot,keta,:)),['-' colors(keta)],'linewidth',1.5);
    end
    hold off
    eta_strings = strtrim(cellstr(num2str(etas(plot_order(:))','%4.2f')));
    if kplot==1
        hy = ylabel('PHA4-variance','fontsize',14);
        ht1 = text(mean(xticks),0.5*yticks(end),sprintf('PHA 4\nA_{%d}',kplot),'fontsize',14);
        set(gca,'ytick',yticks)
   else
        ht1 = text(mean(xticks),0.5*yticks(end),sprintf('PHA 4\nA_{%d}',kplot),'fontsize',14);
   
    end
    set(gca,'xscale','log','yscale','log')
    axis([min_photons max_photons 0 max_y])
    xlabel('number of photons','fontsize',14);
    set(gca,'fontsize',14,'linewidth',1.5,'xtick',xticks)
    hax(end+1) = gca;
end
set(fh,'color','w','position',[250 100 600 1050])

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'MCvarVsCRLB';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% plot increase in variance with eta
load('L:\Projects\Blog\Posts\P52X1SNRpileup5_1\new_code.mat');
fh = figure;
detypes = {'NQ','PHA'};
detnames = {'NQ','PHA 4'};
   % generate CRLB data with more eta values
netas4plot = 15;
etas4plot = linspace(0,0.8,netas4plot);
kt4plot = 3; % index to nphotons to use for plot
nphotons = nphotons2do(kt4plot);
specdat.specnum = nphotons*specdat.specnum/trapz(specdat.egys,specdat.specnum);
NQvars = zeros(netas4plot,2);
PHAvars = zeros(netas4plot,2);
for keta = 1:netas4plot
    eta = etas4plot(keta);
    stats = SNR_with_pileup(specdat, eta);
    NQvars(keta,:) = diag(stats.RA_NQ)';
    RA_NK = CRLB_PHA_with_pileup(specdat,eta);
    PHAvars(keta,:) = diag(RA_NK)';
end
for ktype = 1:2
   subplot(2,1,ktype);
   mcdat = eval(['RA_' detypes{ktype} '_all']);
   crlbdat= eval(['CRLB_' detypes{ktype} '_all']);
   vardat = eval([detypes{ktype} 'vars']);
      % first plot the MC results
   hold on
   plot(etas,squeeze(mcdat(1,1,:,kt4plot)),'ok')
   plot(etas,squeeze(mcdat(2,2,:,kt4plot)),'*r')
   if ktype==1,legend('A_1','A_2','location','northwest');end
         % then the theoretical results
   plot(etas4plot,vardat(:,1),'-k','linewidth',1.5)
   plot(etas4plot,vardat(:,2),'-r','linewidth',1.5)
   hold off
   axis([0 .8 0 4]);
   if ktype==2, xlabel('\eta (photons per dead time)','fontsize',14);end
   text(0.5,3.5,detnames{ktype},'fontsize',14);
   ylabel('variance','fontsize',14);
   set(gca,'fontsize',14,'linewidth',1.5)
end
set(fh,'color','w','position',[250 100 450 850])
%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'VarsVsEta';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',4,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi